#! /bin/bash
echo "INFO: 清理环境"
rm *.rsa
rm *.jks
rm *.p12
rm *.csr
rm *.srl

echo -e "\nINFO: 生成自签发证书"

if [ ! -f ca.key ] || [ ! -f ca.pem ]; then
  echo "INFO: 生成CA根证书"
  openssl req -x509 -new -newkey rsa:2048 -nodes -keyout ca.key -out ca.pem -config ca-openssl.cnf -days 3650 -extensions v3_req
  echo -e "INFO: CA根证书签发完成\n"
  read -t 1
fi

#echo "INFO: Java环境中有keytool命令，生成的ca.jks用于Java中。将 ca.pem 转换为 ca.jks, KeyStore 密码为 123456"
#keytool -importcert -trustcacerts -file ca.pem -keystore ca.jks -storepass 123456
#read -t 1

read -p "INFO: 是否签发服务端证书(PKCS#8 格式) [y/n] " server_yn
if [ "${server_yn}" == "y" ] || [ "${server_yn}" == "Y" ]; then
  echo "INFO: 请填写需要签名的域名和服务器公网IP，签名的域名和IP都可以使用该证书"

  #read -p "请输入根域名(不含www),将为根域名aa.com和所有二级域名签名*.aa.com：" server
  #if [ -n "$server" ];then
  #  echo "INFO: set alt_names $server"
  #  old_server=$(grep "DNS.1 = " server-openssl.cnf|awk -F " " '{print $3}')
  #  echo "INFO: 将 alt_names 从 $old_server 修改为 $server"
  #  sed -i "s/$old_server/$server/g" server-openssl.cnf
  #fi

  alt_names_num=$(awk '/\[alt_names\]/{print NR}' server-openssl.cnf)
  line_start=$(($alt_names_num + 1))
  line_end=$(awk 'END {print NR}' server-openssl.cnf)
  if [ $line_end -gt $line_start ]; then
    echo "INFO: 清空 alt_names "
    sed -i "$line_start,$line_end d" server-openssl.cnf
  fi

  server_domain_yn="y"
  until [ "${server_domain_yn}" == "n" ] || [ "${server_domain_yn}" == "N" ]; do
    read -p "请输入需要签名的域名(www.aa.com 或 *.aa.com 或 aa.com)：" domain
    echo "INFO: 将 $domain 加入到 alt_names"
    dns_num=$(grep -oP "DNS\.\d+\s" server-openssl.cnf|sed 's/[^0-9]//g'|tail -1)
    dns_item=$(($dns_num + 1))
    echo "DNS.$dns_item = $domain">>server-openssl.cnf
    read -p "INFO: 是否还有其他域名需要该证书签名 [y/n] " server_domain_yn
  done

  server_IP_yn="y"
  until [ "${server_IP_yn}" == "n" ] || [ "${server_IP_yn}" == "N" ]; do
    read -p "请输入服务器公网IP：" server_IP
    echo "INFO: 将 $server_IP 加入到 alt_names"
    IP_num=$(grep -oP "IP\.\d+\s" server-openssl.cnf|sed 's/[^0-9]//g'|tail -1)
    IP_item=$(($IP_num + 1))
    echo "IP.$IP_item = $server_IP">>server-openssl.cnf
    read -p "INFO: 是否还有IP需要该证书签名 [y/n] " server_IP_yn
  done

  echo -e "\nINFO: 请检查需要签名的域名和IP是否正确，如若有错，请直接修改 server-openssl.cnf 文件"
  grep -A 50 "\[alt_names\]" server-openssl.cnf
  read -p "按Enter回车键继续"

  openssl genrsa -out server.key.rsa 2048
  openssl pkcs8 -topk8 -in server.key.rsa -out server.key -nocrypt
  openssl req -new -key server.key -out server.csr -config server-openssl.cnf
  openssl x509 -req -CA ca.pem -CAkey ca.key -CAcreateserial -in server.csr -out server.pem -extensions v3_req -extfile server-openssl.cnf -days 3650

  echo "INFO: 将私钥和对应的证书链合成 PKCS#12 格式，KeyStore 密码和私钥密码均为 123456"
  openssl pkcs12 -export -CAfile ca.pem -in server.pem  -inkey server.key -out server.p12 -passout pass:123456
  echo -e "INFO: 服务端证书签发完成\n"
  read -t 1
fi

read -p "INFO: 是否签发反向代理服务器证书(PKCS#1 格式) [y/n] " proxy_client_yn
if [ "${proxy_client_yn}" == "y" ] || [ "${proxy_client_yn}" == "Y" ]; then
  openssl genrsa -out proxy_client.key 2048
  openssl req -new -key proxy_client.key -out proxy_client.csr
  openssl x509 -req -CA ca.pem -CAkey ca.key -CAcreateserial -in proxy_client.csr -out proxy_client.pem -days 3650

  echo "INFO: 将私钥和对应的证书链合成 PKCS#12 格式，KeyStore 密码和私钥密码均为 123456"
  openssl pkcs12 -export -CAfile ca.pem -in proxy_client.pem  -inkey proxy_client.key -out proxy_client.p12 -passout pass:123456
  cat proxy_client.key proxy_client.pem > client_key_crt.pem
  echo -e "INFO: 反向代理服务器证书签发完成\n"
  read -t 1
fi

read -p "INFO: 是否签发客户端证书(PKCS#8 格式) [y/n] " client_yn
if [ "${client_yn}" == "y" ] || [ "${client_yn}" == "Y" ]; then
  echo "INFO: 签发客户端证书(PKCS#8 格式)"
  openssl genrsa -out client.key.rsa 2048
  openssl pkcs8 -topk8 -in client.key.rsa -out client.key -nocrypt
  openssl req -new -key client.key -out client.csr
  openssl x509 -req -CA ca.pem -CAkey ca.key -CAcreateserial -in client.csr -out client.pem -days 3650

  echo "INFO: 将私钥和对应的证书链合成 PKCS#12 格式，KeyStore 密码和私钥密码均为 123456"
  openssl pkcs12 -export -CAfile ca.pem -in client.pem  -inkey client.key -out client.p12 -passout pass:123456
  echo -e "INFO: 客户端证书签发完成\n"
  read -t 1
fi

echo "INFO：清理无用的文件"
rm *.rsa
rm *.csr
rm ca.srl

read -p "Please enter any key to exit" exit_shell
